require "spec_helper"

class DooText
  attr_accessor :zaz
end

describe Nfe::Helpers::XmlModel::AttributeText do
  let(:doo) { DooText.new }

  subject { described_class.new(:zaz) }

  describe "#format" do
    context "when attribute's format is ommited" do
      it "returns the default" do
        expect(subject.format).to eq "%s"
      end
    end
  end

  describe "#parse" do
    let(:doc) { Nokogiri::XML("<zaz>doo_value</zaz>") }

    before { subject.parse doo, doc }

    it "reads like a text attribute" do
      expect(doo.zaz).to eql "doo_value"
    end

    context "when string has scaped little than character (<)" do
      let(:doc) { Nokogiri::XML("<zaz>doo_value &lt;</zaz>") }

      it "reads like a text and convert the original character" do
        expect(doo.zaz).to eql "doo_value <"
      end
    end

    context "when string has scaped great than character (>)" do
      let(:doc) { Nokogiri::XML("<zaz>doo_value &gt;</zaz>") }

      it "reads like a text and convert the original character" do
        expect(doo.zaz).to eql "doo_value >"
      end
    end

    context "when string has scaped & character" do
      let(:doc) { Nokogiri::XML("<zaz>doo_value &amp;</zaz>") }

      it "reads like a text and convert the original character" do
        expect(doo.zaz).to eql "doo_value &"
      end
    end

    context "when string has scaped \" character" do
      let(:doc) { Nokogiri::XML("<zaz>doo_value &quot;</zaz>") }

      it "reads like a text and convert the original character" do
        expect(doo.zaz).to eql "doo_value \""
      end
    end

    context "when string has scaped \' character" do
      let(:doc) { Nokogiri::XML("<zaz>doo_value &#39;</zaz>") }

      it "reads like a text and convert the original character" do
        expect(doo.zaz).to eql "doo_value \'"
      end
    end
  end

  describe "#add_to_xml" do
    let(:xml) { Builder::XmlMarkup.new }

    context "when string has line feed" do
      before { doo.zaz = "doo\nwith line feed" }

      it "returns a text without line feed" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with line feed</zaz>"
      end
    end

    context "when string has carriage return" do
      before { doo.zaz = "doo\rwith carriage return" }

      it "returns a text without carriage return" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with carriage return</zaz>"
      end
    end

    context "when string has more one spaces among the words" do
      before { doo.zaz = "doo  with    a lot of   spaces" }

      it "returns a text with just one space among the words" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with a lot of spaces</zaz>"
      end
    end

    context "when string has little than character (<)" do
      before { doo.zaz = "doo  with <" }

      it "returns a text with scaped <" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with &lt;</zaz>"
      end
    end

    context "when string has great than character (>)" do
      before { doo.zaz = "doo  with >" }

      it "returns a text with scaped >" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with &gt;</zaz>"
      end
    end

    context "when string has the character &" do
      before { doo.zaz = "doo  with &" }

      it "returns a text with scaped &" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with &amp;</zaz>"
      end
    end

    context "when string has the character \"" do
      before { doo.zaz = "doo with \"" }

      it "returns a text with scaped \"" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with &quot;</zaz>"
      end
    end

    context "when string has the character '" do
      before { doo.zaz = "doo with '" }

      it "returns a text with scaped '" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>doo with &#39;</zaz>"
      end
    end

    context "when string has a non ASCII character" do
      before { doo.zaz = "çãé ASCII character – –" }

      it "returns a text without the character" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>çãé ASCII character </zaz>"
      end
    end

    context "when string has a double or more spaces" do
      before { doo.zaz = "text with  space     combinations" }

      it "replaces by a single space" do
        subject.add_to_xml doo, xml, :zaz

        expect(xml.target!).to eq "<zaz>text with space combinations</zaz>"
      end
    end
  end
end
