require "spec_helper"

class Baz
  include ActiveModel::Model
  include Nfe::Helpers::XmlModel::NfeModel
  attr_xml :value
  validates :value, length: { is: 4 }
end

class Base
  include ActiveModel::Model
  include Nfe::Helpers::XmlModel::NfeModel
  extend Enumerize
  enumerize :glu, in: { aaa: 1 }

  def initialize
    @baz = Baz.new
  end

  attr_xml :foo, required: true, xml_alias: :FooAlias
  attr_xml :jaz
  attr_xml :baz, type: :node, required: false
  attr_xml :glu, type: :enumerize
end

class Bar < Base
  attr_xml :float, type: :float, format: "%0.2f"
  attr_xml :integer, type: :integer
end

class Foo < Bar
  attr_xml :date, type: :date
  attr_xml :datetime, type: :datetime
end

describe Nfe::Helpers::XmlModel::NfeModel do
  let(:xml) do
    "<FooAlias>foo</FooAlias>"\
    "<jaz>jaz</jaz>"\
    "<baz><value>baz</value></baz>"\
    "<glu>1</glu>"\
    "<float>10.47</float>"\
    "<integer>28</integer>"\
    "<date>2004-10-24</date>"\
    "<datetime>2004-10-24T02:01:12-03:00</datetime>"
  end

  subject { Foo.new }

  describe ".attr_xml" do
    it "add attribute in class" do
      expect(subject.respond_to?(:foo)).to be
      expect(subject.respond_to?(:jaz)).to be
      expect(subject.respond_to?(:baz)).to be
      expect(subject.respond_to?(:float)).to be
      expect(subject.respond_to?(:integer)).to be
      expect(subject.respond_to?(:date)).to be
      expect(subject.respond_to?(:datetime)).to be
    end
  end

  describe ".xml_attribute" do
    it "get an xml_attribute" do
      expect(subject.class.xml_attribute(:foo)).to be_a Nfe::Helpers::XmlModel::AttributeBase
    end
  end

  describe ".xml_attributes" do
    it "returns all xml_attributes" do
      expect(subject.class.xml_attributes.size).to eql 8
    end
  end

  shared_examples "parse all elements" do
    it "parses all" do
      expect(subject.foo).to eq "foo"
      expect(subject.jaz).to eq "jaz"
      expect(subject.baz.value).to eq "baz"
      expect(subject.glu).to eq "aaa"
      expect(subject.float).to eq 10.47
      expect(subject.integer).to eq 28
      expect(subject.date).to eq Date.parse("2004-10-24")
      expect(subject.datetime).to eq DateTime.strptime("2004-10-24T02:01:12-03:00",
        "%Y-%m-%dT%H:%M:%S%:z")
    end
  end

  describe "#parse!" do
    let(:doc) { "<root>#{xml}</root>" }

    before { subject.parse! doc }

    it_behaves_like "parse all elements"
  end

  describe "#parse_elements" do
    let(:doc) { "<root>#{xml}</root>" }

    context "when the attributes are informed" do
      before do
        subject.parse_elements doc, :foo, :jaz, :baz, :glu, :float, :integer,
          :date, :datetime
      end

      it_behaves_like "parse all elements"
    end

    context "when the attributes aren't informed" do
      before { subject.parse_elements doc }

      it_behaves_like "parse all elements"
    end

    context "when doc is a Nokogiri's object" do
      let(:doc) { Nokogiri::XML("<root>#{xml}</root>") }

      before { subject.parse_elements doc }

      it_behaves_like "parse all elements"
    end
  end

  let(:full_subject) do
    full = Foo.new
    full.foo = "foo"
    full.jaz = "jaz"
    full.baz.value = "baz"
    full.glu = "aaa"
    full.float = 10.47
    full.integer = 28
    full.date = Date.parse("2004-10-24")
    full.datetime = DateTime.strptime("2004-10-24T02:01:12-03:00",
      "%Y-%m-%dT%H:%M:%S%:z")
    full
  end

  describe "#to_xml" do
    it "write all" do
      expect(full_subject.to_xml).to eq xml
    end
  end

  describe "#add_elements" do
    let(:builder) { Builder::XmlMarkup.new }

    shared_examples "write all elements in xml" do
      it "write all" do
        expect(builder.target!).to eq xml
      end
    end

    context "when the attributes are informed" do
      before do
        subject = full_subject
        subject.add_elements builder, :foo, :jaz, :baz, :glu, :float, :integer,
          :date, :datetime
      end

      it_behaves_like "write all elements in xml"
    end

    context "when the attributes aren't informed" do
      before do
        subject = full_subject
        subject.add_elements builder
      end

      it_behaves_like "write all elements in xml"
    end
  end

  describe "#present?" do
    context "when some attribute is present" do
      before { subject.foo = "foo" }
      it "returns true" do
        expect(subject.present?).to be
      end
    end

    context "when any attribute is present" do
      it "returns false" do
        expect(subject.present?).to_not be
      end
    end
  end

  describe "#check_present" do
    context "when some attribute is present" do
      before { subject.foo = "foo" }
      it "returns true" do
        expect(subject.check_present).to be
      end
    end

    context "when any attribute is present" do
      it "returns false" do
        expect(subject.check_present).to_not be
      end
    end
  end

  describe "#validate_optional_attributes" do
    context "when attribute is present and invalid" do
      it "add errors" do
        subject.baz.value = "invalid"
        subject.validate_optional_attributes :baz
        expect(subject.errors.to_json).to match(/baz/)
      end
    end

    context "when attribute isn't present" do
      it "doesn't add errors" do
        subject.validate_optional_attributes :baz
        expect(subject.errors).to be_empty
      end
    end
  end

  describe "#validate_required_attributes" do
    it "add errors when attribute doesn't valid" do
      subject.validate_required_attributes :baz
      expect(subject.errors.to_json).to match(/baz/)
    end
  end
end
