require "spec_helper"

describe Nfe::Models::Det do
  subject { build(:det) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :nItem }
  it { is_expected.to validate_numericality_of(:nItem).only_integer }
  it { is_expected.to ensure_length_of(:nItem).is_at_most(3) }

  it { is_expected.to validate_presence_of :prod }

  it { is_expected.to validate_presence_of :imposto }

  it { is_expected.to allow_value("", nil).for(:infAdProd) }
  it { is_expected.to ensure_length_of(:infAdProd).is_at_most(500) }

  context "when prod is present and invalid" do
    it "returns an error on prod" do
      subject.prod = build(:prod, cfop: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/prod/)
    end
  end

  context "when imposto is present and invalid" do
    it "returns an error on imposto" do
      subject.imposto = build(:imposto)
      subject.imposto.icms.cst = :invalid
      subject.valid?

      expect(subject.errors.to_json).to match(/imposto/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<prod>#{build(:prod).to_xml}</prod>"
      xml += "<imposto>#{build(:imposto).to_xml}</imposto>"
      xml += "<infAdProd>#{"A" * 500}</infAdProd>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Det.new }
      before { subject.parse!("<root nItem='123'>#{xml}</root>") }

      it "parses xml" do
        expect(subject.nItem).to eql "123"
        expect(subject.infAdProd).to eql "A" * 500
        expect(subject.prod.to_xml).to eql build(:prod).to_xml
        expect(subject.imposto.to_xml).to eql build(:imposto).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<prod>#{build(:prod).to_xml}</prod>"
      xml += "<imposto>#{build(:imposto).to_xml}</imposto>"
    end

    describe "#to_xml" do
      subject { build(:det, infAdProd: nil) }
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Det.new }
      before { subject.parse!("<root nItem='123'>#{xml}</root>") }

      it "parses xml" do
        expect(subject.nItem).to eql "123"
        expect(subject.prod.to_xml).to eql build(:prod).to_xml
        expect(subject.imposto.to_xml).to eql build(:imposto).to_xml
      end

      it { is_expected.to be_valid }
    end
  end
end
