require "spec_helper"

describe Nfe::Models::Emitter::Emit do
  subject { build(:emit) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :xNome }
  it { is_expected.to ensure_length_of(:xNome).is_at_least(2).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:xFant) }
  it { is_expected.to ensure_length_of(:xFant).is_at_most(60) }

  it { is_expected.to validate_presence_of :crt }
  it { is_expected.to enumerize(:crt).in(:simple, :exceeded_simple, :normal) }

  it { is_expected.to allow_value("", nil).for(:ieST) }
  it { is_expected.to ensure_length_of(:ieST).is_at_least(2).is_at_most(14) }

  it { is_expected.to allow_value("", nil).for(:im) }
  it { is_expected.to ensure_length_of(:im).is_at_least(2).is_at_most(15) }

  context "when ender is invalid" do
    before { subject.ender.xLgr = "" }

    describe '#valid?' do
      it { is_expected.to_not be_valid }
    end
  end

  describe "#IE" do
    it { is_expected.to validate_presence_of :ie }

    context "when number is invalid" do
      before { subject.ie = "1234" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:ie).is_at_least(2).is_at_most(14) }
  end

  describe "#CPF" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when number is invalid" do
      before { subject.cpf = "11111111122" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cpf).is_equal_to(11) }
  end

  describe "#CNPJ" do
    context "when CPF is present" do
      before { subject.cpf = "11144477735" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14) }
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xNome>Razao Social</xNome>"
      xml += "<xFant>Nome Fantasia</xFant>"
      xml += "<enderEmit>#{build(:ender).to_xml}</enderEmit>"
      xml += "<IE>253263131</IE>"
      xml += "<IEST>253263131</IEST>"
      xml += "<IM>123654</IM>"
      xml += "<CNAE>1234567</CNAE>"
      xml += "<CRT>1</CRT>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Emitter::Emit.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xNome).to eql "Razao Social"
        expect(subject.xFant).to eql "Nome Fantasia"
        expect(subject.ie).to eql "253263131"
        expect(subject.ieST).to eql "253263131"
        expect(subject.im).to eql "123654"
        expect(subject.cnae).to eql "1234567"
        expect(subject.crt).to eql "simple"
        expect(subject.ender.to_json).to eql build(:ender).to_json
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xNome>Razao Social</xNome>"
      xml += "<enderEmit>#{build(:ender).to_xml}</enderEmit>"
      xml += "<IE>253263131</IE>"
      xml += "<CRT>1</CRT>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:emit, xFant: "", ieST: "", im: "", cnae: "")
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Emitter::Emit.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xNome).to eql "Razao Social"
        expect(subject.xFant).to eql nil
        expect(subject.ie).to eql "253263131"
        expect(subject.ieST).to eql nil
        expect(subject.im).to eql nil
        expect(subject.cnae).to eql nil
        expect(subject.crt).to eql "simple"
        expect(subject.ender.to_json).to eql build(:ender).to_json
      end

      it { is_expected.to be_valid }
    end
  end
end
