require "spec_helper"

describe Nfe::Models::EntregaRetirada do
  subject { build(:entrega_retirada) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  describe "#CPF" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when number is invalid" do
      before { subject.cpf = "11111111122" }
      it { is_expected.not_to be_valid }
    end

    it {
      is_expected.to ensure_length_of(:cpf)
        .is_equal_to(11)
        .with_message(/não possui o tamanho esperado/)
    }
  end

  describe "#CNPJ" do
    context "when cpf is present" do
      before { subject.cpf = "11144477735" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it {
      is_expected.to ensure_length_of(:cnpj)
        .is_equal_to(14)
        .with_message(/não possui o tamanho esperado/)
    }
  end

  it { is_expected.to validate_presence_of :xLgr }
  it { is_expected.to ensure_length_of(:xLgr).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :nro }
  it { is_expected.to ensure_length_of(:nro).is_at_most(60) }

  it { is_expected.to allow_value("", nil).for(:xCpl) }
  it { is_expected.to ensure_length_of(:xCpl).is_at_most(60) }

  it { is_expected.to validate_presence_of :xBairro }
  it { is_expected.to ensure_length_of(:xBairro).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :cMun }
  it { is_expected.to ensure_length_of(:cMun).is_equal_to(7) }
  it { is_expected.to validate_numericality_of(:cMun).only_integer }

  it { is_expected.to validate_presence_of :xMun }
  it { is_expected.to ensure_length_of(:xMun).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :uf }
  it { is_expected.to ensure_length_of(:uf).is_equal_to(2) }

  describe "#present?" do
    subject { described_class.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.uf = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xLgr>Rua Geral</xLgr>"
      xml += "<nro>5</nro>"
      xml += "<xCpl>Apartamento</xCpl>"
      xml += "<xBairro>Centro</xBairro>"
      xml += "<cMun>4203709</cMun>"
      xml += "<xMun>Tijucas</xMun>"
      xml += "<UF>SC</UF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xLgr).to eql "Rua Geral"
        expect(subject.nro).to eql "5"
        expect(subject.xCpl).to eql "Apartamento"
        expect(subject.xBairro).to eql "Centro"
        expect(subject.cMun).to eql "4203709"
        expect(subject.xMun).to eql "Tijucas"
        expect(subject.uf).to eql "SC"
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xLgr>Rua Geral</xLgr>"
      xml += "<nro>5</nro>"
      xml += "<xBairro>Centro</xBairro>"
      xml += "<cMun>4203709</cMun>"
      xml += "<xMun>Tijucas</xMun>"
      xml += "<UF>SC</UF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:entrega_retirada, xCpl: "")
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { described_class.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xLgr).to eql "Rua Geral"
        expect(subject.nro).to eql "5"
        expect(subject.xCpl).to be_nil
        expect(subject.xBairro).to eql "Centro"
        expect(subject.cMun).to eql "4203709"
        expect(subject.xMun).to eql "Tijucas"
        expect(subject.uf).to eql "SC"
      end

      it { is_expected.to be_valid }
    end
  end
end
