require "spec_helper"

describe Nfe::Models::Ide do
  subject { build(:ide) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :cUF }
  it { is_expected.to ensure_length_of(:cUF).is_equal_to(2)}

  it { is_expected.to validate_presence_of :cNF }
  it { is_expected.to validate_numericality_of(:cNF) }
  it { is_expected.to ensure_length_of(:cNF).is_at_most(8)}

  it { is_expected.to validate_presence_of :natOp }
  it { is_expected.to ensure_length_of(:natOp).is_at_most(60)}

  it { is_expected.to validate_presence_of :indPag }
  it { is_expected.to enumerize(:indPag).in(:cash_payment, :installment_payment, :others) }

  it { is_expected.to validate_presence_of :idDest }
  it { is_expected.to enumerize(:idDest).in(:state, :interstate, :external) }

  it { is_expected.to validate_presence_of :serie }
  it { is_expected.to validate_numericality_of(:serie) }
  it { is_expected.to ensure_length_of(:serie).is_at_most(3)}

  it { is_expected.to validate_presence_of :nNF }
  it { is_expected.to validate_numericality_of(:nNF) }
  it { is_expected.to ensure_length_of(:nNF).is_at_most(9)}

  it { is_expected.to validate_presence_of :dhEmi }

  it { is_expected.to validate_presence_of :tpNF }
  it { is_expected.to enumerize(:tpNF).in(:entry, :departure) }

  it { is_expected.to validate_presence_of :cMunFG }
  it { is_expected.to ensure_length_of(:cMunFG).is_at_most(7)}

  it { is_expected.to validate_presence_of :tpImp }
  it { is_expected.to enumerize(:tpImp).in(:portrait, :landscape) }

  it { is_expected.to validate_presence_of :tpEmis }
  it { is_expected.to enumerize(:tpEmis).in(:normal, :fs, :scan, :dpec, :fs_da, :svc_an,
    :svc_rs) }

  it { is_expected.to validate_presence_of :cDV }
  it { is_expected.to validate_numericality_of(:cDV) }
  it { is_expected.to ensure_length_of(:cDV).is_equal_to(1)}

  it { is_expected.to validate_presence_of :tpAmb }
  it { is_expected.to enumerize(:tpAmb).in(:production, :test) }

  it { is_expected.to validate_presence_of :finNFe }
  it { is_expected.to enumerize(:finNFe).in(:normal, :complement, :adjustment, :returning) }

  it { is_expected.to validate_presence_of :verProc }
  it { is_expected.to ensure_length_of(:verProc).is_at_most(20)}

  context "when NFref is present and invalid" do
    it "returns an error on NFref" do
      subject.NFref << build(:NFref, refNFe: "A"*45)
      subject.valid?

      expect(subject.errors.to_json).to match(/NFref/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<cUF>42</cUF>"
      xml += "<cNF>12345678</cNF>"
      xml += "<natOp>5101-Venda de producao do estabelecimento</natOp>"
      xml += "<indPag>1</indPag>"
      xml += "<mod>55</mod>"
      xml += "<serie>123</serie>"
      xml += "<nNF>123456789</nNF>"
      xml += "<dhEmi>2013-09-17T13:00:15-03:00</dhEmi>"
      xml += "<dhSaiEnt>2013-09-18T02:01:12-03:00</dhSaiEnt>"
      xml += "<tpNF>1</tpNF>"
      xml += "<idDest>1</idDest>"
      xml += "<cMunFG>1234567</cMunFG>"
      xml += "<tpImp>1</tpImp>"
      xml += "<tpEmis>1</tpEmis>"
      xml += "<cDV>1</cDV>"
      xml += "<tpAmb>1</tpAmb>"
      xml += "<finNFe>1</finNFe>"
      xml += "<indFinal>0</indFinal>"
      xml += "<indPres>9</indPres>"
      xml += "<procEmi>0</procEmi>"
      xml += "<verProc>1</verProc>"
      xml += "<dhCont>2013-09-18T02:01:12-03:00</dhCont>"
      xml += "<xJust>#{"A" * 15}</xJust>"

      xml += "<NFref>#{build(:NFref, refNFe: "#{"1" * 44}", refCTe: nil,
        refNF: nil, refNFP: nil, refECF: nil).to_xml}</NFref>"

      xml += "<NFref>#{build(:NFref, refCTe: "#{"2" * 44}", refNFe: nil,
        refNF: nil, refNFP: nil, refECF: nil).to_xml}</NFref>"

      xml += "<NFref>#{build(:NFref, refNFe: nil, refCTe: nil, refNFP: nil,
        refECF: nil).to_xml}</NFref>"

      xml += "<NFref>#{build(:NFref, refNFe: nil, refCTe: nil, refNF: nil,
        refECF: nil).to_xml}</NFref>"

      xml += "<NFref>#{build(:NFref, refCTe: nil, refNFe: nil, refNF: nil,
       refNFP: nil).to_xml}</NFref>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Ide.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.uf).to eql "SC"
        expect(subject.cNF).to eql "12345678"
        expect(subject.natOp).to eql "5101-Venda de producao do estabelecimento"
        expect(subject.indPag).to eql "installment_payment"
        expect(subject.serie).to eql "123"
        expect(subject.nNF).to eql "123456789"
        expect(subject.dhEmi).to eql DateTime.strptime("2013-09-17T13:00:15-03:00", "%Y-%m-%dT%H:%M:%S%:z")
        expect(subject.dhSaiEnt).to eql DateTime.strptime("2013-09-18T02:01:12-03:00", "%Y-%m-%dT%H:%M:%S%:z")
        expect(subject.tpNF).to eql "departure"
        expect(subject.idDest).to eql "state"
        expect(subject.cMunFG).to eql "1234567"
        expect(subject.tpImp).to eql "portrait"
        expect(subject.tpEmis).to eql "normal"
        expect(subject.cDV).to eql "1"
        expect(subject.tpAmb).to eql "production"
        expect(subject.finNFe).to eql "normal"
        expect(subject.verProc).to eql "1"
        expect(subject.xJust).to eql "A" * 15

        expect(subject.dhCont).to eql DateTime.strptime("2013-09-18T02:01:12-03:00",
          "%Y-%m-%dT%H:%M:%S%:z")

        expect(subject.NFref[0].refNFe).to eql "1" * 44
        expect(subject.NFref[1].refCTe).to eql "2" * 44
        expect(subject.NFref[2].refNF.to_xml).to eql build(:refNF).to_xml
        expect(subject.NFref[3].refNFP.to_xml).to eql build(:refNFP).to_xml
        expect(subject.NFref[4].refECF.to_xml).to eql build(:refECF).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<cUF>42</cUF>"
      xml += "<cNF>12345678</cNF>"
      xml += "<natOp>5101-Venda de producao do estabelecimento</natOp>"
      xml += "<indPag>1</indPag>"
      xml += "<mod>55</mod>"
      xml += "<serie>123</serie>"
      xml += "<nNF>123456789</nNF>"
      xml += "<dhEmi>2013-09-17T13:00:15-03:00</dhEmi>"
      xml += "<tpNF>1</tpNF>"
      xml += "<idDest>1</idDest>"
      xml += "<cMunFG>1234567</cMunFG>"
      xml += "<tpImp>1</tpImp>"
      xml += "<tpEmis>1</tpEmis>"
      xml += "<cDV>1</cDV>"
      xml += "<tpAmb>1</tpAmb>"
      xml += "<finNFe>1</finNFe>"
      xml += "<indFinal>0</indFinal>"
      xml += "<indPres>9</indPres>"
      xml += "<procEmi>0</procEmi>"
      xml += "<verProc>1</verProc>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build :ide, dhSaiEnt: nil, xJust: nil, dhCont: nil
        subject.NFref.clear
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Ide.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.uf).to eql "SC"
        expect(subject.cNF).to eql "12345678"
        expect(subject.natOp).to eql "5101-Venda de producao do estabelecimento"
        expect(subject.indPag).to eql "installment_payment"
        expect(subject.serie).to eql "123"
        expect(subject.nNF).to eql "123456789"
        expect(subject.dhEmi).to eql DateTime.strptime("2013-09-17T13:00:15-03:00", "%Y-%m-%dT%H:%M:%S%z")
        expect(subject.tpNF).to eql "departure"
        expect(subject.idDest).to eql "state"
        expect(subject.cMunFG).to eql "1234567"
        expect(subject.tpImp).to eql "portrait"
        expect(subject.tpEmis).to eql "normal"
        expect(subject.cDV).to eql "1"
        expect(subject.tpAmb).to eql "production"
        expect(subject.finNFe).to eql "normal"
        expect(subject.verProc).to eql "1"
      end

      it { is_expected.to be_valid }
    end
  end
end
