require "spec_helper"

describe Nfe::Models::InfNFe do
  subject { build(:infNFe) }

  context "when ide is invalid" do
    it "returns a error on ide" do
      subject.ide.tpNF = 3
      subject.valid?

      expect(subject.errors.to_json).to match(/ide/)
    end
  end

  context "when emit is invalid" do
    it "returns a error on emit" do
      subject.emit.xNome = "A"
      subject.valid?

      expect(subject.errors.to_json).to match(/emit/)
    end
  end

  context "when dest is invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:dest) { Nfe::Models::Recipient::Dest.new }

    it "returns a error on dest" do
      dest.xNome = "A"
      subject.dest = dest

      subject.valid?

      expect(subject.errors.to_json).to match(/dest/)
    end
  end

  context "when total is invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:total) { Nfe::Models::Total.new }

    it "returns a error on total" do
      total.icmsTot.vBC = 999_999_999_999_999.99
      subject.total = total

      subject.valid?

      expect(subject.errors.to_json).to match(/total/)
    end
  end

  context "when transp is invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:transp) { Nfe::Models::Transport::Transp.new }

    it "returns a error on transp" do
      transp.modFrete = 99
      subject.transp = transp

      subject.valid?

      expect(subject.errors.to_json).to match(/transp/)
    end
  end

  context "when avulsa is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:avulsa) { Nfe::Models::Avulsa.new }

    it "returns a error on avulsa" do
      avulsa.uf = "ABC"
      subject.avulsa = avulsa

      subject.valid?

      expect(subject.errors.to_json).to match(/avulsa/)
    end
  end

  context "when retirada is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:retirada) { Nfe::Models::EntregaRetirada.new }

    it "returns a error on retirada" do
      retirada.cpf = "1"
      subject.retirada = retirada

      subject.valid?

      expect(subject.errors.to_json).to match(/retirada/)
    end
  end

  context "when entrega is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:entrega) { Nfe::Models::EntregaRetirada.new }

    it "returns a error on entrega" do
      entrega.cpf = "1"
      subject.entrega = entrega

      subject.valid?

      expect(subject.errors.to_json).to match(/entrega/)
    end
  end

  context "when cobr is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:cobr) { Nfe::Models::Bill::Cobr.new }

    it "returns a error on cobr" do
      cobr.fat.vOrig = 999_999_999_999_999.99
      subject.cobr = cobr

      subject.valid?

      expect(subject.errors.to_json).to match(/cobr/)
    end
  end

  context "when infAdic is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:infAdic) { Nfe::Models::Information::InfAdic.new }

    it "returns a error on infAdic" do
      infAdic.infAdFisco = "A" * 2500
      subject.infAdic = infAdic

      subject.valid?

      expect(subject.errors.to_json).to match(/infAdic/)
    end
  end

  context "when exporta is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:exporta) { Nfe::Models::Exporta.new }

    it "returns a error on exporta" do
      exporta.ufEmbarq = "A"
      subject.exporta = exporta

      subject.valid?

      expect(subject.errors.to_json).to match(/exporta/)
    end
  end

  context "when compra is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:compra) { Nfe::Models::Compra.new }

    it "returns a error on compra" do
      compra.xCont = "A" * 70
      subject.compra = compra

      subject.valid?

      expect(subject.errors.to_json).to match(/compra/)
    end
  end

  context "when cana is present and invalid" do
    subject { Nfe::Models::InfNFe.new }
    let(:cana) { Nfe::Models::Cane::Cana.new }

    it "returns a error on cana" do
      cana.vFor = "A"
      subject.cana = cana

      subject.valid?

      expect(subject.errors.to_json).to match(/cana/)
    end
  end

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:versao).is_at_least(1).is_at_most(4) }
  
  context "when xml has all tags" do
    let(:xml) do
      xml = "<ide>#{build(:ide).to_xml}</ide>"
      xml += "<emit>#{build(:emit).to_xml}</emit>"
      xml += "<avulsa>#{build(:avulsa).to_xml}</avulsa>"
      xml += "<dest>#{build(:dest).to_xml}</dest>"
      xml += "<retirada>#{build(:entrega_retirada).to_xml}</retirada>"
      xml += "<entrega>#{build(:entrega_retirada).to_xml}</entrega>"
      xml += "<det nItem=\"1\">#{build(:det, ).to_xml}</det>"
      xml += "<det nItem=\"2\">#{build(:det, ).to_xml}</det>"
      xml += "<total>#{build(:total).to_xml}</total>"
      xml += "<transp>#{build(:transp).to_xml}</transp>"
      xml += "<cobr>#{build(:cobr).to_xml}</cobr>"
      xml += "<infAdic>#{build(:infAdic).to_xml}</infAdic>"
      xml += "<exporta>#{build(:exporta).to_xml}</exporta>"
      xml += "<compra>#{build(:compra).to_xml}</compra>"
      xml += "<cana>#{build(:cana).to_xml}</cana>"
    end

    describe "#id" do
      it "returns infNFe id" do
        expect(subject.id).to eq "NFe42130901144012000152551231234567891123456785"
      end
    end

    describe "#key" do
      it "returns nfe key" do
        expect(subject.key).to eq "42130901144012000152551231234567891123456785"
      end
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::InfNFe.new }
      before { subject.parse!("<root versao=\"v2.0\" Id=\"NFe42130901144012000152551231234567891123456785\">#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.versao).to eql "v2.0"
        expect(subject.id).to eql "NFe42130901144012000152551231234567891123456785"
        expect(subject.ide.to_xml).to eql build(:ide).to_xml
        expect(subject.emit.to_xml).to eql build(:emit).to_xml
        expect(subject.avulsa.to_xml).to eql build(:avulsa).to_xml
        expect(subject.dest.to_xml).to eql build(:dest).to_xml
        expect(subject.retirada.to_xml).to eql build(:entrega_retirada).to_xml
        expect(subject.entrega.to_xml).to eql build(:entrega_retirada).to_xml
        expect(subject.det[0].to_xml).to eql build(:det, nItem: "1").to_xml
        expect(subject.det[1].to_xml).to eql build(:det, nItem: "2").to_xml
        expect(subject.total.to_xml).to eql build(:total).to_xml
        expect(subject.transp.to_xml).to eql build(:transp).to_xml
        expect(subject.cobr.to_xml).to eql build(:cobr).to_xml
        expect(subject.exporta.to_xml).to eql build(:exporta).to_xml
        expect(subject.compra.to_xml).to eql build(:compra).to_xml
        expect(subject.cana.to_xml).to eql build(:cana).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
     let(:xml) do
      xml = "<ide>#{build(:ide).to_xml}</ide>"
      xml += "<emit>#{build(:emit).to_xml}</emit>"
      xml += "<dest>#{build(:dest).to_xml}</dest>"
      xml += "<det nItem=\"1\">#{build(:det, nItem: "1").to_xml}</det>"
      xml += "<det nItem=\"2\">#{build(:det, nItem: "2").to_xml}</det>"
      xml += "<total>#{build(:total).to_xml}</total>"
      xml += "<transp>#{build(:transp).to_xml}</transp>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:infNFe, avulsa: nil, retirada: nil, entrega: nil, cobr: nil,
          infAdic: nil, compra: nil, exporta:nil, cana: nil, cobr: nil)
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::InfNFe.new }
      before { subject.parse!("<root versao=\"v2.0\" Id=\"NFe42130901144012000152551231234567891123456785\">#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.versao).to eql "v2.0"
        expect(subject.id).to eql "NFe42130901144012000152551231234567891123456785"
        expect(subject.ide.to_xml).to eql build(:ide).to_xml
        expect(subject.emit.to_xml).to eql build(:emit).to_xml
        expect(subject.avulsa).to_not be_present
        expect(subject.dest.to_xml).to eql build(:dest).to_xml
        expect(subject.retirada).to_not be_present
        expect(subject.entrega).to_not be_present
        expect(subject.det[0].to_xml).to eql build(:det, nItem: "1").to_xml
        expect(subject.det[1].to_xml).to eql build(:det, nItem: "2").to_xml
        expect(subject.total.to_xml).to eql build(:total).to_xml
        expect(subject.transp.to_xml).to eql build(:transp).to_xml
        expect(subject.cobr).to_not be_present
        expect(subject.exporta).to_not be_present
        expect(subject.cana).to_not be_present
      end

      it { is_expected.to be_valid }
    end
  end
end
