require "spec_helper"

describe Nfe::Models::Itens::Prod do
  subject { build(:prod) }

  context "when ncm is present and invalid" do
    it "returns an error on ncm" do
      subject.ncm = "123456"
      subject.valid?

      expect(subject.errors.to_json).to match(/ncm/)
    end
  end

  context "when cean is present and invalid" do
    it "returns an error on cean" do
      subject.cEAN = "123456"
      subject.valid?

      expect(subject.errors.to_json).to match(/cEAN/)
    end
  end

  context "when cEANTrib is present and invalid" do
    it "returns an error on cEANTrib" do
      subject.cEANTrib = "123456"
      subject.valid?

      expect(subject.errors.to_json).to match(/cEANTrib/)
    end
  end

  context "when DI is present and invalid" do
    it "returns an error on DI" do
      subject.di << build(:di, ufDesemb: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/di/)
    end
  end

  context "when veicProd is present and invalid" do
    it "returns an error on veicProd" do
      subject.veicProd = build(:veicProd, tpOp: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/veicProd/)
    end
  end

  context "when comb is present and invalid" do
    it "returns an error on comb" do
      subject.comb = build(:comb, cProdANP: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/comb/)
    end
  end

  context "when med is present and invalid" do
    it "returns an error on med" do
      subject.med << build(:med, qLote: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/med/)
    end
  end

  context "when arma is present and invalid" do
    it "returns an error on arma" do
      subject.arma << build(:arma, tpArma: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/arma/)
    end
  end

  it { is_expected.to ensure_length_of(:cProd).is_at_least(1).is_at_most(60) }

  it { is_expected.to ensure_length_of(:xProd).is_at_least(1).is_at_most(120) }

  it { is_expected.to allow_value("", nil).for(:extipi) }
  it { is_expected.to ensure_length_of(:extipi).is_at_least(2).is_at_most(3) }

  it { is_expected.to ensure_length_of(:cfop).is_equal_to(4) }

  it { is_expected.to ensure_length_of(:uCom).is_at_least(1).is_at_most(6) }

  it { is_expected.to validate_numericality_of(:qCom) }

  it { is_expected.to validate_numericality_of(:vUnCom) }

  it { is_expected.to validate_numericality_of(:vProd) }

  it { is_expected.to ensure_length_of(:uTrib).is_at_least(1).is_at_most(6) }

  it { is_expected.to validate_numericality_of(:qTrib) }

  it { is_expected.to validate_numericality_of(:vUnTrib) }

  it { is_expected.to validate_numericality_of(:vFrete) }

  it { is_expected.to validate_numericality_of(:vSeg) }

  it { is_expected.to validate_numericality_of(:vDesc) }

  it { is_expected.to validate_numericality_of(:vOutro) }

  it { is_expected.to ensure_length_of(:indTot).is_equal_to(1) }

  it { is_expected.to allow_value("", nil).for(:xPed) }
  it { is_expected.to ensure_length_of(:xPed).is_at_most(15) }

  it { is_expected.to allow_value("", nil).for(:nItemPed) }
  it { is_expected.to ensure_length_of(:nItemPed).is_equal_to(6) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  context "when xml has all tags" do
    let(:xml) do
      xml =  "<cProd>1</cProd>"
      xml += "<cEAN>111444772735</cEAN>"
      xml += "<xProd>Nome do produto</xProd>"
      xml += "<NCM>02084000</NCM>"
      xml += "<EXTIPI>12</EXTIPI>"
      xml += "<CFOP>1101</CFOP>"
      xml += "<uCom>Un</uCom>"
      xml += "<qCom>99999999999.99</qCom>"
      xml += "<vUnCom>99999999999.99</vUnCom>"
      xml += "<vProd>99999999999.99</vProd>"
      xml += "<cEANTrib>12345678</cEANTrib>"
      xml += "<uTrib>Un</uTrib>"
      xml += "<qTrib>99999999999.99</qTrib>"
      xml += "<vUnTrib>99999999999.99</vUnTrib>"
      xml += "<vFrete>99999999999.99</vFrete>"
      xml += "<vSeg>99999999999.99</vSeg>"
      xml += "<vDesc>99999999999.99</vDesc>"
      xml += "<vOutro>99999999999.99</vOutro>"
      xml += "<indTot>0</indTot>"
      2.times { |n| xml += "<DI>#{build(:di, nDI: "12X3ABC4#{n}").to_xml}</DI>" }
      xml += "<xPed>1234</xPed>"
      xml += "<nItemPed>000001</nItemPed>"
      xml += "<veicProd>#{build(:veicProd).to_xml}</veicProd>"
      3.times { |n| xml += "<med>#{build(:med, nLote: "12345678#{n}").to_xml}</med>" }
      2.times { |n| xml += "<arma>#{build(:arma, nSerie: "12345#{n}").to_xml}</arma>" }
      xml += "<comb>#{build(:comb).to_xml}</comb>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Prod.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.cProd).to eql "1"
        expect(subject.cEAN).to eql "111444772735"
        expect(subject.xProd).to eql "Nome do produto"
        expect(subject.ncm).to eql "02084000"
        expect(subject.extipi).to eql "12"
        expect(subject.cfop).to eql "1101"
        expect(subject.uCom).to eql "Un"
        expect(subject.qCom).to eql "99999999999.99"
        expect(subject.vUnCom).to eql "99999999999.99"
        expect(subject.vProd).to eql 99999999999.99
        expect(subject.cEANTrib).to eql "12345678"
        expect(subject.uTrib).to eql "Un"
        expect(subject.qTrib).to eql "99999999999.99"
        expect(subject.vUnTrib).to eql "99999999999.99"
        expect(subject.vFrete).to eql 99999999999.99
        expect(subject.vSeg).to eql 99999999999.99
        expect(subject.vDesc).to eql 99999999999.99
        expect(subject.vOutro).to eql 99999999999.99
        expect(subject.indTot).to eql "0"
        expect(subject.di[0].to_xml).to eql build(:di, nDI: "12X3ABC40").to_xml
        expect(subject.di[1].to_xml).to eql build(:di, nDI: "12X3ABC41").to_xml
        expect(subject.xPed).to eql "1234"
        expect(subject.nItemPed).to eql "000001"
        expect(subject.veicProd.to_xml).to eql build(:veicProd).to_xml
        expect(subject.med[0].to_xml).to eql build(:med, nLote: "123456780").to_xml
        expect(subject.med[1].to_xml).to eql build(:med, nLote: "123456781").to_xml
        expect(subject.med[2].to_xml).to eql build(:med, nLote: "123456782").to_xml
        expect(subject.arma[0].to_xml).to eql build(:arma, nSerie: "123450").to_xml
        expect(subject.arma[1].to_xml).to eql build(:arma, nSerie: "123451").to_xml
        expect(subject.comb.to_xml).to eql build(:comb).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { build(:prod, extipi: nil, cEANTrib: nil, cEAN: nil, vFrete: nil,
      xPed: nil, nItemPed: nil, vSeg: nil, vDesc: nil, vOutro: nil) }

    let(:xml) do
      xml =  "<cProd>1</cProd>"
      xml += "<cEAN/>"
      xml += "<xProd>Nome do produto</xProd>"
      xml += "<NCM>02084000</NCM>"
      xml += "<CFOP>1101</CFOP>"
      xml += "<uCom>Un</uCom>"
      xml += "<qCom>99999999999.99</qCom>"
      xml += "<vUnCom>99999999999.99</vUnCom>"
      xml += "<vProd>99999999999.99</vProd>"
      xml += "<cEANTrib/>"
      xml += "<uTrib>Un</uTrib>"
      xml += "<qTrib>99999999999.99</qTrib>"
      xml += "<vUnTrib>99999999999.99</vUnTrib>"
      xml += "<indTot>0</indTot>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject.di = []
        subject.med = []
        subject.arma = []
        subject.veicProd = Nfe::Models::Itens::VeicProd.new
        subject.comb = Nfe::Models::Itens::Comb.new
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Itens::Prod.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.extipi).to_not be_present
        expect(subject.cEAN).to be_blank
        expect(subject.cEANTrib).to be_blank
        expect(subject.vFrete).to_not be_present
        expect(subject.vSeg).to_not be_present
        expect(subject.vDesc).to_not be_present
        expect(subject.vOutro).to_not be_present
      end

      it { is_expected.to be_valid }
    end
  end
end
