require "spec_helper"

describe Nfe::Models::Recipient::Dest do
  subject { build(:dest) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:xNome).is_at_least(2).is_at_most(60) }

  it { is_expected.to validate_presence_of :indIEDest }
  it { is_expected.to enumerize(:indIEDest).in(:taxpayer, :not_taxpayer, :immune) }

  describe "#IE" do
    context "when number is invalid" do
      before { subject.ie = "1234" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:ie).is_at_most(14) }
    it { is_expected.to allow_value("", nil).for(:ie) }
  end

  it { is_expected.to allow_value("", nil).for(:isuf) }
  it { is_expected.to ensure_length_of(:isuf).is_at_least(8).is_at_most(9) }

  describe "#email" do
    context "when number is invalid" do
      before { subject.email = "invalid" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:email).is_at_most(60) }
    it { is_expected.to allow_value("", nil).for(:email) }
  end

  describe "#CPF" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when ender.uf is 'EX' (External)" do
      before { subject.ender.uf = "EX" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when number is invalid" do
      before { subject.cpf = "11111111122" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cpf).is_equal_to(11) }
  end

  describe "#CNPJ" do
    context "when cpf is present" do
      before { subject.cpf = "11144477735" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when ender.uf is 'EX' (External)" do
      before { subject.ender.uf = "EX" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cnpj).is_equal_to(14) }
  end

  context "when ender is invalid" do
    before { subject.ender.xLgr = "" }
    it { is_expected.to_not be_valid }
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xNome>Razao Social</xNome>"
      xml += "<enderDest>#{build(:ender).to_xml}</enderDest>"
      xml += "<indIEDest>1</indIEDest>"
      xml += "<IE>253263131</IE>"
      xml += "<ISUF>123456789</ISUF>"
      xml += "<email>jon@doe.com</email>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Recipient::Dest.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xNome).to eql "Razao Social"
        expect(subject.indIEDest).to eql "taxpayer"
        expect(subject.ie).to eql "253263131"
        expect(subject.isuf).to eql "123456789"
        expect(subject.email).to eql "jon@doe.com"
        expect(subject.ender.to_json).to eql build(:ender).to_json
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    let(:xml) do
      xml  = "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<xNome>Razao Social</xNome>"
      xml += "<enderDest>#{build(:ender).to_xml}</enderDest>"
      xml += "<indIEDest>1</indIEDest>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        subject = build(:dest, ie: nil, isuf: nil, email: "")
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Recipient::Dest.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.xNome).to eql "Razao Social"
        expect(subject.indIEDest).to eql "taxpayer"
        expect(subject.ie).to be_blank
        expect(subject.isuf).to be_blank
        expect(subject.email).to be_blank
        expect(subject.ender.to_json).to eql build(:ender).to_json
      end

      it { is_expected.to be_valid }
    end
  end
end
