require "spec_helper"

describe Nfe::Models::Referenced::RefNFP do
  subject { build(:refNFP) }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to ensure_length_of(:cUF).is_equal_to(2) }

  it { is_expected.to ensure_length_of(:aamm).is_equal_to(4) }

  it { is_expected.to ensure_length_of(:mod).is_equal_to(2) }

  it { is_expected.to ensure_length_of(:serie).is_at_least(1).is_at_most(3) }

  it { is_expected.to ensure_length_of(:nNF).is_at_least(1).is_at_most(6) }

  describe "#CPF" do
    context "when CNPJ is present" do
      before { subject.cnpj = "01144012000152" }
      it { is_expected.to allow_value("").for(:cpf) }
    end

    context "when number is invalid" do
      before { subject.cpf = "11111111122" }
      it { is_expected.not_to be_valid }
    end

    it {
      is_expected.to ensure_length_of(:cpf)
        .is_equal_to(11)
        .with_message(/não possui o tamanho esperado/)
    }
  end

  describe "#CNPJ" do
    context "when cpf is present" do
      before { subject.cpf = "11144477735" }
      it { is_expected.to allow_value("").for(:cnpj) }
    end

    context "when number is invalid" do
      before { subject.cnpj = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it {
      is_expected.to ensure_length_of(:cnpj)
        .is_equal_to(14)
        .with_message(/não possui o tamanho esperado/)
    }
  end

  describe "#IE" do
    context "when number is invalid" do
      before { subject.ie = "1234" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:ie).is_at_least(2).is_at_most(14) }
  end

  describe "#present?" do
    subject { Nfe::Models::Referenced::RefNFP.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.uf = "SC" }

      it { is_expected.to be_present }
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml = "<cUF>42</cUF>"
      xml += "<AAMM>1302</AAMM>"
      xml += "<CNPJ>01144012000152</CNPJ>"
      xml += "<CPF>11144477735</CPF>"
      xml += "<IE>253263131</IE>"
      xml += "<mod>04</mod>"
      xml += "<serie>123</serie>"
      xml += "<nNF>123456</nNF>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Referenced::RefNFP.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.uf).to eql "SC"
        expect(subject.aamm).to eql "1302"
        expect(subject.cnpj ).to eql "01144012000152"
        expect(subject.cpf).to eql "11144477735"
        expect(subject.ie).to eql "253263131"
        expect(subject.mod).to eql "04"
        expect(subject.serie).to eql "123"
        expect(subject.nNF).to eql "123456"
      end

      it { is_expected.to be_valid }
    end
  end
end
