require "spec_helper"

describe Nfe::Models::Taxes::IPI do

  subject { build(:ipi_when_Trib, cst: "50") }

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to allow_value("", nil).for(:clEnq) }
  it { is_expected.to ensure_length_of(:clEnq).is_equal_to(5) }

  describe "#CNPJProd" do
    it { is_expected.to allow_value("").for(:cnpjProd) }

    context "when number is invalid" do
      before { subject.cnpjProd = "04001155000103" }
      it { is_expected.not_to be_valid }
    end

    it { is_expected.to ensure_length_of(:cnpjProd).is_equal_to(14) }
  end

  it { is_expected.to ensure_length_of(:cSelo).is_at_most(60) }

  it { is_expected.to ensure_length_of(:qSelo).is_at_most(12) }

  it { is_expected.to validate_presence_of(:cst) }
  it { is_expected.to validate_inclusion_of(:cst).in_array(["01", "02", "03", "04", "51",
    "52", "53", "54", "55", "99"]) }

  describe "#present?" do
    subject { Nfe::Models::Taxes::IPI.new }

    context "when any attribute is present" do
      it { is_expected.to_not be_present }
    end

    context "when some attribute is present" do
      before { subject.vBC = 1 }

      it { is_expected.to be_present }
    end
  end

  context "when IPI has a IPITrib node" do
    subject { build(:ipi_when_Trib, cst: "50") }

    context "when xml has all tags" do
      let(:xml) do
        xml  = "<clEnq>12345</clEnq>"
        xml += "<CNPJProd>01144012000152</CNPJProd>"
        xml += "<cSelo>#{"A" * 60}</cSelo>"
        xml += "<qSelo>123456789012</qSelo>"
        xml += "<cEnq>999</cEnq>"
        xml += "<IPITrib>#{build(:ipiTrib, cst:"50").to_xml}</IPITrib>"
      end

      describe "#to_xml" do
        it "returns a formatted xml" do
          expect(subject.to_xml).to eql xml
        end
      end

      describe "#parse!" do
        subject { Nfe::Models::Taxes::IPI.new }
        before { subject.parse!("<root>#{xml}</root>") }

        it "parses xml values to itself" do
          expect(subject.clEnq).to eql "12345"
          expect(subject.cnpjProd).to eql "01144012000152"
          expect(subject.cSelo).to eql "A" * 60
          expect(subject.qSelo).to eql "123456789012"
          expect(subject.cEnq).to eql "999"
          expect(subject.cst).to eql "50"
          expect(subject.vBC).to eql 12345.67
          expect(subject.pIPI).to eql 988.76
          expect(subject.qUnid).to eql 123.66
          expect(subject.vUnid).to eql 78.65
          expect(subject.vIPI).to eql 23456.78
        end

        it { is_expected.to be_valid }
      end
    end
  end

  context "when IPI has a IPINT node" do
    subject do
      build(:ipi_when_NT, clEnq: nil, cnpjProd: nil, cSelo: nil, qSelo: nil)
    end

    context "when xml doesn't have optional tags" do
      let(:xml) do
        xml  = "<cEnq>999</cEnq>"
        xml += "<IPINT>#{build(:ipiNT, cst:"01").to_xml}</IPINT>"
      end

      describe "#to_xml" do
        it "returns a formatted xml" do
          expect(subject.to_xml).to eql xml
        end
      end

      describe "#parse!" do
        subject { Nfe::Models::Taxes::IPI.new }
        before { subject.parse!("<root>#{xml}</root>") }

        it "parses xml values to itself" do
          expect(subject.clEnq).to be_blank
          expect(subject.cnpjProd).to be_blank
          expect(subject.cSelo).to be_blank
          expect(subject.qSelo).to be_blank
          expect(subject.vBC).to be_blank
          expect(subject.pIPI).to be_blank
          expect(subject.qUnid).to be_blank
          expect(subject.vUnid).to be_blank
          expect(subject.vIPI).to be_blank
          expect(subject.cEnq).to eql "999"
          expect(subject.cst).to eql "01"
        end

        it { is_expected.to be_valid }
      end
    end
  end
end
