require "spec_helper"

describe Nfe::Models::Total do
  subject { build(:total) }

  context "when ICMSTot is present and invalid" do
    it "returns a error on ICMSTot" do
      subject.icmsTot.vBC = 999_999_999_999_999.99
      subject.valid?

      expect(subject.errors.to_json).to match(/icmsTot/)
    end
  end

  context "when ISSQNtot is present and invalid" do
    it "returns a error on ISSQNtot" do
      subject.issqntot.vServ = 999_999_999_999_999.99
      subject.valid?

      expect(subject.errors.to_json).to match(/issqntot/)
    end
  end

  context "when retTrib is present and invalid" do
    it "returns an error on retTrib" do
      subject.retTrib.vRetPIS = 999_999_999_999_999.99
      subject.valid?

      expect(subject.errors.to_json).to match(/retTrib/)
    end
  end

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :icmsTot }

  it { is_expected.to allow_value("", nil).for(:issqntot) }

  it { is_expected.to allow_value("", nil).for(:retTrib) }

  context "when xml has all tags" do
    let(:xml) do
      xml = "<ICMSTot>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<vICMSDeson>23456.78</vICMSDeson>"
      xml += "<vBCST>34567.89</vBCST>"
      xml += "<vST>45678.90</vST>"
      xml += "<vProd>12345.67</vProd>"
      xml += "<vFrete>23456.78</vFrete>"
      xml += "<vSeg>34567.89</vSeg>"
      xml += "<vDesc>45678.90</vDesc>"
      xml += "<vII>91234.56</vII>"
      xml += "<vIPI>78901.23</vIPI>"
      xml += "<vPIS>45678.90</vPIS>"
      xml += "<vCOFINS>89123.45</vCOFINS>"
      xml += "<vOutro>67890.12</vOutro>"
      xml += "<vNF>34567.89</vNF>"
      xml += "<vTotTrib>100.45</vTotTrib>"
      xml += "</ICMSTot>"
      xml += "<ISSQNtot>"
      xml += "<vServ>12345.67</vServ>"
      xml += "<vBC>23456.78</vBC>"
      xml += "<vISS>34567.89</vISS>"
      xml += "<vPIS>91234.56</vPIS>"
      xml += "<vCOFINS>78912.34</vCOFINS>"
      xml += "</ISSQNtot>"
      xml += "<retTrib>"
      xml += "<vRetPIS>12345.67</vRetPIS>"
      xml += "<vRetCOFINS>23456.78</vRetCOFINS>"
      xml += "<vRetCSLL>34567.89</vRetCSLL>"
      xml += "<vBCIRRF>91234.56</vBCIRRF>"
      xml += "<vIRRF>78912.34</vIRRF>"
      xml += "<vBCRetPrev>56789.01</vBCRetPrev>"
      xml += "<vRetPrev>67890.12</vRetPrev>"
      xml += "</retTrib>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Total.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.icmsTot.to_xml).to eql build(:icmsTot).to_xml
        expect(subject.issqntot.to_xml).to eql build(:issqntot).to_xml
        expect(subject.retTrib.to_xml).to eql build(:retTrib).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { build(:total, issqntot: nil, retTrib: nil) }
    before { subject.icmsTot.vTotTrib = nil }

    let(:xml) do
      xml = "<ICMSTot>"
      xml += "<vBC>12345.67</vBC>"
      xml += "<vICMS>23456.78</vICMS>"
      xml += "<vICMSDeson>23456.78</vICMSDeson>"
      xml += "<vBCST>34567.89</vBCST>"
      xml += "<vST>45678.90</vST>"
      xml += "<vProd>12345.67</vProd>"
      xml += "<vFrete>23456.78</vFrete>"
      xml += "<vSeg>34567.89</vSeg>"
      xml += "<vDesc>45678.90</vDesc>"
      xml += "<vII>91234.56</vII>"
      xml += "<vIPI>78901.23</vIPI>"
      xml += "<vPIS>45678.90</vPIS>"
      xml += "<vCOFINS>89123.45</vCOFINS>"
      xml += "<vOutro>67890.12</vOutro>"
      xml += "<vNF>34567.89</vNF>"
      xml += "</ICMSTot>"
      xml
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Total.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml" do
        expect(subject.icmsTot.to_xml).to eql build(:icmsTot, vTotTrib: nil).to_xml
      end

      it { is_expected.to be_valid }
    end
  end
end
