require "spec_helper"

describe Nfe::Models::Transport::Transp do
  subject { build(:transp) }

  context "when transporta is present and invalid" do
    it "returns an error on transporta" do
      subject.transporta.cpf = "11111111111"
      subject.valid?

      expect(subject.errors.to_json).to match(/transporta/)
    end
  end

  context "when rettransp is present and invalid" do
    it "returns an error on retTransp" do
      subject.retTransp.cfop = "11111"
      subject.valid?

      expect(subject.errors.to_json).to match(/retTransp/)
    end
  end

  context "when veictransp is present and invalid" do
    it "returns an error on veicTransp" do
      subject.veicTransp.uf = "AAA"
      subject.valid?

      expect(subject.errors.to_json).to match(/veicTransp/)
    end
  end

  context "when reboque is present and invalid" do
    it "returns an error on reboque" do
      subject.reboque[0].uf = "AAA"
      subject.valid?

      expect(subject.errors.to_json).to match(/reboque/)
    end
  end

  context "when vol is present and invalid" do
    it "returns an error on vol" do
      subject.vol << build(:vol, qVol: :invalid)
      subject.valid?

      expect(subject.errors.to_json).to match(/vol/)
    end
  end

  it "has a valid factory" do
    expect(subject).to be_valid
  end

  it { is_expected.to validate_presence_of :modFrete }
  it { is_expected.to enumerize(:modFrete).in(:emitter, :recipient,
    :third_party, :no_freight) }

  it { is_expected.to allow_value("", nil).for(:vagao) }
  it { is_expected.to ensure_length_of(:vagao).is_at_most(20) }

  it { is_expected.to allow_value("", nil).for(:balsa) }
  it { is_expected.to ensure_length_of(:balsa).is_at_most(20) }

  describe "#reboque" do
    it "should be less than or equal to 5" do
      6.times { subject.reboque << build(:veicTranspReboque) }
      subject.valid?
      expect(subject.errors[:reboque].to_s).to match(/deve ser menor ou igual a 5/)
    end
  end

  context "when xml has all tags" do
    let(:xml) do
      xml  = "<modFrete>1</modFrete>"
      xml += "<transporta>#{build(:transporta).to_xml}</transporta>"
      xml += "<retTransp>#{build(:retTransp).to_xml}</retTransp>"
      xml += "<veicTransp>#{build(:veicTranspReboque).to_xml}</veicTransp>"
      xml += "<reboque>#{build(:veicTranspReboque, placa: "AAA1110").to_xml}</reboque>"
      xml += "<reboque>#{build(:veicTranspReboque, placa: "AAA1111").to_xml}</reboque>"
      xml += "<vagao>#{"V" * 20}</vagao>"
      xml += "<balsa>#{"B" * 20}</balsa>"
      xml += "<vol>#{build(:vol, qVol: 0).to_xml}</vol>"
      xml += "<vol>#{build(:vol, qVol: 1).to_xml}</vol>"
    end

    describe "#to_xml" do
      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end
    end

    describe "#parse!" do
      subject { Nfe::Models::Transport::Transp.new }
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.modFrete).to eql "recipient"
        expect(subject.vagao).to eql "V" * 20
        expect(subject.balsa).to eql "B" * 20
        expect(subject.transporta.to_xml).to eql build(:transporta).to_xml
        expect(subject.retTransp.to_xml).to eql build(:retTransp).to_xml
        expect(subject.veicTransp.to_xml).to eql build(:veicTranspReboque).to_xml

        expect(subject.reboque[0].to_xml).to eql build(:veicTranspReboque,
          placa: "AAA1110").to_xml

        expect(subject.reboque[1].to_xml).to eql build(:veicTranspReboque,
          placa: "AAA1111").to_xml

        expect(subject.vol[0].to_xml).to eql build(:vol, qVol: 0).to_xml
        expect(subject.vol[1].to_xml).to eql build(:vol, qVol: 1).to_xml
      end

      it { is_expected.to be_valid }
    end
  end

  context "when xml doesn't have optional tags" do
    subject { Nfe::Models::Transport::Transp.new }
    let(:xml) { "<modFrete>1</modFrete>" }


    describe "#to_xml" do
      before { subject.modFrete = 1 }

      it "returns a formatted xml" do
        expect(subject.to_xml).to eql xml
      end

      it { is_expected.to be_valid }
    end

    describe "#parse!" do
      before { subject.parse!("<root>#{xml}</root>") }

      it "parses xml values to itself" do
        expect(subject.modFrete).to eql "recipient"
        expect(subject.balsa).to be_blank
        expect(subject.vagao).to be_blank
        expect(subject.transporta).to_not be_present
        expect(subject.retTransp).to_not be_present
        expect(subject.veicTransp).to_not be_present
      end

      it { is_expected.to be_valid }
    end
  end
end
