require "spec_helper"
require_relative "shared_examples_for_operations"

describe Nfe::Operations::CorrectionLetter do
  let(:private_key) { OpenSSL::PKey::RSA.new(File.read("spec/nfe/fixtures/private-key.pem")) }
  let(:certificate) { OpenSSL::X509::Certificate.new(File.read("spec/nfe/fixtures/certificate.pem")) }
  let(:nfe_key) { "42150303883369000188550010000000301000000308" }
  let(:date_time) { DateTime.new(2015, 3, 6, 14, 49, 59, "-3") }
  let(:correction) { "correction letter test" }

  subject do
    described_class.new(certificate: certificate, private_key: private_key,
      environment: :test, nfe_key: nfe_key, sequential: 1, date_time: date_time,
      correction: correction)
  end

  describe "#execute" do
    context "when works" do
      it "returns true" do
        VCR.use_cassette("correction_letter/success") do
          expect(subject.execute).to be_truthy
        end
      end
    end

    context "when certificate is invalid" do
      let(:certificate) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_certificate")
      end
    end

    context "when private_key is invalid" do
      let(:private_key) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_private_key")
      end
    end

    context "when nfe_key is invalid" do
      let(:nfe_key) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("xml.invalid_nfe_key")
      end
    end

    context "when correction is invalid" do
      let(:correction) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("correction_letter.invalid_correction")
      end
    end

    context "when webservice informs that is an invalid lot" do
      before { VCR.use_cassette("correction_letter/error_in_lot") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end

    context "when result xml is unexpected" do
      before { VCR.use_cassette("correction_letter/invalid_response") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end
  end

  describe "#success?" do
    context "when SEFAZ returns rejected" do
      before { VCR.use_cassette("correction_letter/rejected") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_success
      end
    end

    context "when SEFAZ returns success" do
      before { VCR.use_cassette("correction_letter/success") { subject.execute } }

      it "returns true" do
        expect(subject).to be_success
      end
    end
  end

  describe "#rejected?" do
    context "when SEFAZ returns reject" do
      before { VCR.use_cassette("correction_letter/rejected") { subject.execute } }

      it "returns true" do
        expect(subject).to be_rejected
      end
    end

    context "when SEFAZ doesn't reject" do
      before { VCR.use_cassette("correction_letter/success") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_rejected
      end
    end
  end

  describe "#processed?" do
    ["success", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("correction_letter/#{state}") { subject.execute } }

        it "returns true" do
          expect(subject).to be_processed
        end

        it "doesn't add messages in errors" do
          expect(subject.errors).to be_empty
        end
      end
    end

    context "when SEFAZ doesn't return a valid state" do
      before { VCR.use_cassette("correction_letter/error_in_lot") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_processed
      end
    end
  end

  describe "#message" do
    context "when SEFAZ doesn't return a NFe's state" do
      before { VCR.use_cassette("correction_letter/error_in_lot") { subject.execute } }

      it "returns nil" do
        expect(subject.message).to be_nil
      end
    end

    ["success", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("correction_letter/#{state}") { subject.execute } }

        it "returns the SEFAZ message" do
          expect(subject.message).to_not be_nil
        end
      end
    end
  end

  describe "#correction_letter_xml" do
    context "when success" do
      before { VCR.use_cassette("correction_letter/success") { subject.execute } }

      it "returns the XML" do
        expect(subject.correction_letter_xml).to start_with "<procEventoNFe"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("correction_letter/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.correction_letter_xml).to be_blank
      end
    end
  end

  describe "#protocol_number" do
    context "when canceled" do
      before { VCR.use_cassette("correction_letter/success") { subject.execute } }

      it "returns the protocol number" do
        expect(subject.protocol_number).to eq "342150000157864"
      end
    end

    context "when not canceled" do
      before { VCR.use_cassette("correction_letter/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.protocol_number).to be_blank
      end
    end
  end
end
