require "spec_helper"
require_relative "shared_examples_for_operations"

describe Nfe::Operations::NfeCancel do
  let(:private_key) { OpenSSL::PKey::RSA.new(File.read("spec/nfe/fixtures/private-key.pem")) }
  let(:certificate) { OpenSSL::X509::Certificate.new(File.read("spec/nfe/fixtures/certificate.pem")) }
  let(:nfe_key) { "42150201144012000152550010000000321000000329" }
  let(:date_time) { DateTime.new(2015, 2, 18, 9, 2, 59, "-3") }
  let(:protocol) { "342150000069725" }
  let(:reason) { "example of justification" }

  subject do
    described_class.new(certificate, private_key, environment: :test,
      nfe_key: nfe_key, date_time: date_time, protocol: protocol,
      reason: reason)
  end

  describe "#execute" do
    context "when works" do
      it "returns true" do
        VCR.use_cassette("nfe_cancel/success") do
          expect(subject.execute).to be_truthy
        end
      end
    end

    context "when certificate is invalid" do
      let(:certificate) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_certificate")
      end
    end

    context "when private_key is invalid" do
      let(:private_key) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_private_key")
      end
    end

    context "when nfe_key is invalid" do
      let(:nfe_key) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("xml.invalid_nfe_key")
      end
    end

    context "when protocol is invalid" do
      let(:protocol) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_cancel.invalid_protocol")
      end
    end

    context "when reason is invalid" do
      let(:reason) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_cancel.invalid_reason")
      end
    end

    context "when webservice informs that is an invalid lot" do
      before { VCR.use_cassette("nfe_cancel/error_in_lot") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end

    context "when result xml is unexpected" do
      before { VCR.use_cassette("nfe_cancel/invalid_response") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end
  end

  describe "#success?" do
    context "when SEFAZ doesn't cancel" do
      before { VCR.use_cassette("nfe_cancel/rejected") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_success
      end
    end

    context "when SEFAZ returns canceled" do
      before { VCR.use_cassette("nfe_cancel/success") { subject.execute } }

      it "returns true" do
        expect(subject).to be_success
      end
    end
  end

  describe "#rejected?" do
    context "when SEFAZ returns reject" do
      before { VCR.use_cassette("nfe_cancel/rejected") { subject.execute } }

      it "returns true" do
        expect(subject).to be_rejected
      end
    end

    context "when SEFAZ doesn't reject" do
      before { VCR.use_cassette("nfe_cancel/success") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_rejected
      end
    end
  end

  describe "#processed?" do
    ["success", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_cancel/#{state}") { subject.execute } }

        it "returns true" do
          expect(subject).to be_processed
        end

        it "doesn't add messages in errors" do
          expect(subject.errors).to be_empty
        end
      end
    end

    context "when is an invalid cancellation statement" do
      subject do
        described_class.new(certificate, private_key, environment: :test,
          nfe_key: nfe_key, date_time: date_time, protocol: "invalid",
          reason: "invalid")
      end

      before { subject.execute }

      it "fills an errors object" do
        expect(subject).to_not be_processed
      end
    end

    context "when SEFAZ doesn't return a cancel state" do
      before { VCR.use_cassette("nfe_cancel/error_in_lot") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_processed
      end
    end
  end

  describe "#message" do
    context "when SEFAZ doesn't return a NFe's state" do
      before { VCR.use_cassette("nfe_cancel/error_in_lot") { subject.execute } }

      it "returns nil" do
        expect(subject.message).to be_nil
      end
    end

    ["success", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_cancel/#{state}") { subject.execute } }

        it "returns the SEFAZ message" do
          expect(subject.message).to_not be_nil
        end
      end
    end
  end

  describe "#canceled_xml" do
    context "when success" do
      before { VCR.use_cassette("nfe_cancel/success") { subject.execute } }

      it "returns the XML" do
        expect(subject.canceled_xml).to start_with "<procEventoNFe"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("nfe_cancel/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.canceled_xml).to be_blank
      end
    end
  end

  describe "#protocol_number" do
    context "when canceled" do
      before { VCR.use_cassette("nfe_cancel/success") { subject.execute } }

      it "returns the protocol number" do
        expect(subject.protocol_number).to eq "342150000069742"
      end
    end

    context "when not canceled" do
      before { VCR.use_cassette("nfe_cancel/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.protocol_number).to be_blank
      end
    end
  end
end
