require "spec_helper"
require_relative "shared_examples_for_operations"

describe Nfe::Operations::NfeDiscard do
  let(:private_key) { OpenSSL::PKey::RSA.new(File.read("spec/nfe/fixtures/private-key.pem")) }
  let(:certificate) { OpenSSL::X509::Certificate.new(File.read("spec/nfe/fixtures/certificate.pem")) }
  let(:reason) { "example of justification" }
  let(:uf) { "SC" }
  let(:cnpj) { "03883369000188" }
  let(:series) { 1 }
  let(:initial_number) { 1100 }

  subject do
    described_class.new(certificate: certificate, private_key: private_key,
      environment: :test, uf: uf, cnpj: cnpj, series: series, reason: reason,
      initial_number: initial_number)
  end

  describe "#execute" do
    context "when works" do
      it "returns true" do
        VCR.use_cassette("nfe_discard/success") do
          expect(subject.execute).to be_truthy
        end
      end
    end

    context "when certificate is invalid" do
      let(:certificate) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_certificate")
      end
    end

    context "when private_key is invalid" do
      let(:private_key) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("webservice.invalid_private_key")
      end
    end

    context "when uf is invalid" do
      let(:uf) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_discard.invalid_uf")
      end
    end

    context "when cnpj is invalid" do
      let(:cnpj) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_discard.invalid_cnpj")
      end
    end

    context "when series is invalid" do
      let(:series) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_discard.invalid_series")
      end
    end

    context "when number is invalid" do
      let(:initial_number) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_discard.invalid_number")
      end
    end

    context "when reason is invalid" do
      let(:reason) { nil }

      before { subject.execute }

      it "fills an errors object" do
        expect(subject.errors.full_messages).to include I18n.t("nfe_discard.invalid_reason")
      end
    end

    context "when result xml is unexpected" do
      before { VCR.use_cassette("nfe_discard/invalid_response") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end
  end

  describe "#success?" do
    context "when SEFAZ rejects the issue" do
      before { VCR.use_cassette("nfe_discard/rejected") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_success
      end
    end

    context "when SEFAZ returns discarded" do
      before { VCR.use_cassette("nfe_discard/success") { subject.execute } }

      it "returns true" do
        expect(subject).to be_success
      end
    end
  end

  describe "#rejected?" do
    context "when SEFAZ returns reject" do
      before { VCR.use_cassette("nfe_discard/rejected") { subject.execute } }

      it "returns true" do
        expect(subject).to be_rejected
      end
    end

    context "when SEFAZ doesn't reject" do
      before { VCR.use_cassette("nfe_discard/success") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_rejected
      end
    end
  end

  describe "#message" do
    ["success", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_discard/#{state}") { subject.execute } }

        it "returns the SEFAZ message" do
          expect(subject.message).to_not be_nil
        end
      end
    end
  end

  describe "#discard_xml" do
    context "when success" do
      before { VCR.use_cassette("nfe_discard/success") { subject.execute } }

      it "returns the XML" do
        expect(subject.discard_xml).to start_with "<ProcInutNFe"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("nfe_discard/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.discard_xml).to be_blank
      end
    end
  end
end
