require "spec_helper"

describe Nfe::Operations::NfeSend do
  let(:private_key) { OpenSSL::PKey::RSA.new(File.read("spec/nfe/fixtures/private-key.pem")) }
  let(:certificate) { OpenSSL::X509::Certificate.new(File.read("spec/nfe/fixtures/certificate.pem")) }

  let(:nfe) do
    nfe = Nfe::Models::NFe.new
    nfe.parse!(File.read("spec/nfe/fixtures/nfe.xml"))
    nfe
  end

  subject { described_class.new(certificate, private_key, nfe) }

  describe "#execute" do
    context "when works" do
      it "returns true" do
        VCR.use_cassette("nfe_send/authorized") do
          expect(subject.execute).to be_truthy
        end
      end
    end

    context "when webservice informs that is an invalid lot" do
      before { VCR.use_cassette("nfe_send/error_in_lot") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end

    context "when result xml is unexpected" do
      before { VCR.use_cassette("nfe_send/invalid_response") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end

    context "when nfe is invalid" do
      let(:nfe) do
        nfe = Nfe::Models::NFe.new
        nfe.parse!(File.read("spec/nfe/fixtures/nfe.xml"))
        nfe.infNFe.emit.xNome = nil
        nfe
      end

      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "fills an errors object" do
        expect(subject.errors).to_not be_empty
      end
    end
  end

  describe "#nfe_status" do
    ["authorized", "denied", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_send/#{state}") { subject.execute } }

        it "returns #{state}" do
          expect(subject.nfe_status).to be state.to_sym
        end
      end
    end

    context "when webservice doesn't process the NFe" do
      before { VCR.use_cassette("nfe_send/error_in_lot") { subject.execute } }

      it "returns false" do
        expect(subject.nfe_status).to eq :pending
      end
    end
  end


  describe "#authorized?" do
    context "when SEFAZ doesn't authorize" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_authorized
      end
    end

    context "when SEFAZ returns authorized" do
      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "returns true" do
        expect(subject).to be_authorized
      end
    end
  end

  describe "#denied?" do
    context "when SEFAZ returns denied" do
      before { VCR.use_cassette("nfe_send/denied") { subject.execute } }

      it "returns true" do
        expect(subject).to be_denied
      end
    end

    context "when SEFAZ doesn't denied" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_denied
      end
    end
  end

  describe "#rejected?" do
    context "when SEFAZ returns reject" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns true" do
        expect(subject).to be_rejected
      end
    end

    context "when SEFAZ doesn't reject" do
      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "returns false" do
        expect(subject).to_not be_rejected
      end
    end
  end

  describe "#processed?" do
    ["authorized", "denied", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_send/#{state}") { subject.execute } }

        it "returns true" do
          expect(subject).to be_processed
        end

        it "doesn't add messages in errors" do
          expect(subject.errors).to be_empty
        end
      end
    end
  end

  describe "#message" do
    context "when SEFAZ doesn't return a NFe's state" do
      before { VCR.use_cassette("nfe_send/error_in_lot") { subject.execute } }

      it "returns nil" do
        expect(subject.message).to be_nil
      end
    end

    ["authorized", "denied", "rejected"].each do |state|
      context "when SEFAZ returns #{state}" do
        before { VCR.use_cassette("nfe_send/#{state}") { subject.execute } }

        it "returns the SEFAZ message" do
          expect(subject.message).to_not be_nil
        end
      end
    end
  end

  describe "#authorized_xml" do
    context "when authorized" do
      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "returns the XML" do
        expect(subject.authorized_xml).to start_with "<nfeProc"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.authorized_xml).to be_blank
      end
    end
  end

  describe "#protocol_number" do
    context "when authorized" do
      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "returns the protocol number" do
        expect(subject.protocol_number).to eq "342140000245084"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.protocol_number).to be_blank
      end
    end
  end

  describe "#nfe_key" do
    context "when authorized" do
      before { VCR.use_cassette("nfe_send/authorized") { subject.execute } }

      it "returns the nfe key" do
        expect(subject.nfe_key).to eq "42150101144012000152550010000001111000000734"
      end
    end

    context "when not authorized" do
      before { VCR.use_cassette("nfe_send/rejected") { subject.execute } }

      it "returns blank" do
        expect(subject.nfe_key).to be_blank
      end
    end
  end
end
