require 'spec_helper'

describe Xmldsig::SignedDocument do
  let(:signed_xml) { File.read("spec/xmldsig/fixtures/signed.xml") }
  let(:signed_document) { Xmldsig::SignedDocument.new(signed_xml) }
  let(:unsigned_xml) { File.read("spec/xmldsig/fixtures/unsigned.xml") }
  let(:unsigned_document) { Xmldsig::SignedDocument.new(unsigned_xml) }
  let(:private_key) { OpenSSL::PKey::RSA.new(File.read("spec/xmldsig/fixtures/key.pem")) }
  let(:certificate) { OpenSSL::X509::Certificate.new(File.read("spec/xmldsig/fixtures/certificate.cer")) }
  let(:other_certificate) { OpenSSL::X509::Certificate.new(File.read("spec/xmldsig/fixtures/certificate2.cer")) }

  describe "#initialize" do
    it "sets the document to a nokogiri document" do
      document = described_class.new(signed_xml)
      expect(document.document).to be_a(Nokogiri::XML::Document)
    end
  end

  describe "#signatures" do
    let(:unsigned_xml) { File.read("spec/xmldsig/fixtures/unsigned_nested_signature.xml") }
    let(:unsigned_document) { Xmldsig::SignedDocument.new(unsigned_xml) }

    it "returns only the signed nodes" do
      expect(signed_document.signatures).to be_all { |signature| signature.is_a?(Xmldsig::Signature) }
    end

    it "returns the nested signatures first" do
      expect(unsigned_document.signatures.first.references.first.reference_uri).to eql "#baz"
    end
  end

  describe "#signed_nodes" do
    it "returns only the signed nodes" do
      expect(signed_document.signed_nodes.collect(&:name)).to eql %w(Foo)
    end
  end

  describe "#validate" do
    it "returns true if the signature and digest value are correct" do
      expect(signed_document.validate(certificate)).to be_truthy
    end

    it "returns false if the certificate is not valid" do
      expect(signed_document.validate(other_certificate)).to be_falsey
    end

    it "returns false if there are no signatures and validation is strict" do
      xml_without_signature = Xmldsig::SignedDocument.new('<foo></foo>')
      expect(xml_without_signature.validate(certificate)).to be_falsey
    end

    it "accepts a block" do
      expect(
        signed_document.validate do |signature_value, data|
          certificate.public_key.verify(OpenSSL::Digest::SHA256.new, signature_value, data)
        end
      ).to be_truthy
    end
  end

  describe "#sign" do
    it "returns a signed document" do
      signed_document = unsigned_document.sign(private_key, certificate)
      expect(Xmldsig::SignedDocument.new(signed_document).
        validate(certificate)).to be_truthy
    end

    it "accepts a block" do
      signed_document = unsigned_document.sign do |data|
        private_key.sign(OpenSSL::Digest::SHA256.new, data)
      end
      expect(Xmldsig::SignedDocument.new(signed_document).
        validate(certificate)).to be_truthy
    end
  end

  describe "Nested Signatures" do
    let(:unsigned_xml) { File.read("spec/xmldsig/fixtures/unsigned_nested_signature.xml") }
    let(:unsigned_document) { Xmldsig::SignedDocument.new(unsigned_xml) }
    let(:signed_document) { unsigned_document.sign(private_key, certificate) }

    it "when signed should be valid" do
      expect(Xmldsig::SignedDocument.new(signed_document).
        validate(certificate)).to be_truthy
    end

    it "should sign 2 elements" do
      expect(unsigned_document.signed_nodes.count).to eql 2
    end

    it "allows individual signs" do
      unsigned_document.signatures.last.sign(private_key, certificate)
      expect(unsigned_document.validate(certificate)).to be_falsey
      expect(unsigned_document.signatures.last.valid?(certificate)).to be_truthy
    end
  end
end
